/*
 * MapPanel.java
 *
 * Created on 8 mars 2007, 16:55
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package net.tinyos.mapviewer;


import javax.swing.*;

import java.awt.*;              //for layout managers and more
import java.awt.geom.*;

/**
 *
 * @author dwarfy
 */
public class MapPanel extends JPanel {
    private int fontSize = 12;
    
    Stroke drawingStroke = new BasicStroke(Config.LINE_WIDTH_AXIS);
    double zoomAct;
    boolean zoomChanged,displayGrid,drawingNodes,drawHistory;
    
    Stabilisator S;
    
    /** Creates a new instance of MapPanel */
    public MapPanel() {
        
    zoomAct= 0.5;
    zoomChanged=true;
    displayGrid=true;
    drawingNodes=false;
    drawHistory=false;
    
    
    Font font = new Font("Serif", Font.PLAIN, fontSize);
    setFont(font);
    FontMetrics metrics = getFontMetrics(font);

    
    }
    public void zoomIn() {
        if (zoomAct-Config.ZOOM_STEP >= Config.ZOOM_MIN) {
            zoomAct -= Config.ZOOM_STEP;
            zoomChanged=true;
            repaint();
        }
    }
    public void zoomOut() {
        if (zoomAct+Config.ZOOM_STEP <= Config.ZOOM_MAX) {
            zoomAct += Config.ZOOM_STEP;
            zoomChanged=true;
            repaint();
        }
    }
   
    public void setStab(Stabilisator Stab) {
        S = Stab;
    }
    
    public void setDrawingNodes(boolean drawing) {
        drawingNodes=drawing;
    }
    
    public void paintComponent(Graphics g) {
        if (zoomChanged) {
            setPreferredSize(new Dimension((int)(Config.XMAX*zoomAct),(int)(Config.YMAX*zoomAct)));
            revalidate();
            zoomChanged=false;
        }
        
        super.paintComponent(g);

        drawGrids(g);
        drawAnchors(g);
        if (drawingNodes) {
            if (drawHistory) {
                drawNodesH(g);
            } else {
                drawNodes(g);
            }
            
        }
        
    }
    
    public void drawGrids(Graphics g) {
        //draw grid        
        if (displayGrid == true) {
            g.setColor(Color.GRAY);
            int xorig = (int)((Config.XMAX*zoomAct)/2);
            int yorig = (int)((Config.YMAX*zoomAct)/2);
            for (int x=-Config.XMAX+Config.RESO;x<Config.XMAX;x+=Config.RESO) {
                g.drawLine((int)(xorig+(x*zoomAct)),0,
                    (int)(xorig+(x*zoomAct)),(int)(Config.YMAX*zoomAct));
            }
            for (int y=-Config.YMAX+Config.RESO;y<Config.YMAX;y+=Config.RESO) {
                g.drawLine(0,(int)(yorig+(y*zoomAct)),
                    (int)(Config.XMAX*zoomAct),(int)(yorig+(y*zoomAct)));                
            }
        }
        
        
        //draw axis
        g.setColor(Color.BLACK);        
        Graphics2D g2d = (Graphics2D)g;
        Line2D line1 = new Line2D.Float();
        Line2D line2 = new Line2D.Float();
        
        g2d.setStroke(drawingStroke);
        
        line1.setLine((float)((Config.XMAX*zoomAct)/2),0f,
                    (float)((Config.XMAX*zoomAct)/2),(float)(Config.YMAX*zoomAct));
        
        line2.setLine(0f,(float)((Config.YMAX*zoomAct)/2),
                    (float)(Config.XMAX*zoomAct),(float)((Config.YMAX*zoomAct)/2));
        
        g2d.draw(line1);
        g2d.draw(line2);
        
     
    }
    
    public void drawNodesH(Graphics g) {
        Coord[] path;
        int iix=0,iiy=0,iix2=0,iiy2=0,iiw=1;
        
        for (short x=0;x<Config.NUM_NODES;x++) {
            path = S.getPath(x);
            
            for (int y=0;y<Config.NODE_PATH_SIZE&&y<path.length;y++) {
                
                iix = (int)( (path[y].x*zoomAct) + ((Config.XMAX*zoomAct)/2) );
                iiy = (int)( (path[y].y*zoomAct) + ((Config.YMAX*zoomAct)/2) );

                iiw = (int)( zoomAct * Config.MAX_NODE_WIDTH );
                if (iiw < 1)iiw=1;
            
                g.setColor(Color.RED);
                g.fillOval(iix,iiy,iiw,iiw);
                if (y+1<Config.NODE_PATH_SIZE) {
                    iix2 = (int)( (path[y+1].x*zoomAct) + ((Config.XMAX*zoomAct)/2) );
                    iiy2 = (int)( (path[y+1].y*zoomAct) + ((Config.YMAX*zoomAct)/2) );
                    g.drawLine(iix,iiy,iix2,iiy2);
                }
                
            }
            
        
        }
        
        
        
     
        
    
    }
    
       public void drawNodes(Graphics g) {
        Coord[] poss;
        int iix=0,iiy=0,iix2=0,iiy2=0,iiw=1;
        
        poss = S.getLastPaths();
        
        Graphics2D g2d = (Graphics2D)g;
        
        for (short x=0;x<Config.NUM_NODES;x++) {
            iix = (int)( (poss[x].x*zoomAct) + ((Config.XMAX*zoomAct)/2) );
            iiy = (int)( (poss[x].y*zoomAct) + ((Config.YMAX*zoomAct)/2) );

            iiw = (int)( zoomAct * Config.MAX_NODE_WIDTH );
            if (iiw < 1)iiw=1;

            g.setColor(Color.RED);
            g.fillOval(iix,iiy,iiw,iiw);

            
             g2d.setPaint(Color.black);
             g2d.drawString(String.valueOf(Config.NODES_IDS[x]), iix, iiy);
        
        }
        
  
    }
    
    
   
    
    public void drawAnchors(Graphics g) {
        int iix=0,iiy=0,iiw=1;
                Graphics2D g2d = (Graphics2D)g;
        //draw anchor nodes
        for (int i=0;i<Config.NUM_ANCHORS;i++) {
            iix = (int)( (Config.ANCHORS_POS[i][0]*zoomAct) + ((Config.XMAX*zoomAct)/2) );
            iiy = (int)( (Config.ANCHORS_POS[i][1]*zoomAct) + ((Config.YMAX*zoomAct)/2) );

            iiw = (int)( zoomAct * Config.MAX_NODE_WIDTH );
            if (iiw < 1)iiw=1;
            
            g.setColor(Color.BLUE);
            g.fillOval(iix,iiy,iiw,iiw);
        
          /*  System.out.println("Anchor"+i+
                "zoomAct "+ zoomAct +
                "Config.ZOOM_MAX "+ Config.ZOOM_MAX +
                "iix "+ iix +
                "iiy "+ iiy +
                "iiw "+ iiw);    */  
            
                        
             g2d.setPaint(Color.black);
             g2d.drawString(String.valueOf(Config.ANCHORS_IDS[i]), iix, iiy);
            
        }            
    }
}
